@@ -35,6 +35,7 @@
 #include <dbus/dbus-hash.h>
 #include <dbus/dbus-timeout.h>
 #include <dbus/dbus-connection-internal.h>
+#include <dbus/dbus-internals.h>
 
 /* Trim executed commands to this length; we want to keep logs readable */
 #define MAX_LOG_COMMAND_LEN 50
@@ -134,6 +135,7 @@ connection_get_loop (DBusConnection *connection)
   BusConnectionData *d;
 
   d = BUS_CONNECTION_DATA (connection);
+  _dbus_assert(d != NULL);
 
   return bus_context_get_loop (d->connections->context);
 }
@@ -638,9 +640,12 @@ static void
 check_pending_fds_cb (DBusConnection *connection)
 {
   BusConnectionData *d = BUS_CONNECTION_DATA (connection);
-  int n_pending_unix_fds_old = d->n_pending_unix_fds;
+  int n_pending_unix_fds_old;
   int n_pending_unix_fds_new;
 
+  _dbus_assert(d != NULL);
+
+  n_pending_unix_fds_old = d->n_pending_unix_fds;
   n_pending_unix_fds_new = _dbus_connection_get_pending_fds_count (connection);
 
   _dbus_verbose ("Pending fds count changed on connection %p: %d -> %d\n",
@@ -666,6 +671,32 @@ static dbus_bool_t
 pending_unix_fds_timeout_cb (void *data)
 {
   DBusConnection *connection = data;
+  BusConnectionData *d = BUS_CONNECTION_DATA (connection);
+  unsigned long uid;
+  int limit;
+
+  _dbus_assert (d != NULL);
+  limit = bus_context_get_pending_fd_timeout (d->connections->context);
+
+  if (dbus_connection_get_unix_user (connection, &uid) && uid == 0)
+    {
+      bus_context_log (d->connections->context, DBUS_SYSTEM_LOG_WARNING,
+                       "Connection \"%s\" (%s) has had Unix fds pending for "
+                       "too long (pending_fd_timeout=%dms); tolerating it, "
+                       "because it has uid 0",
+                       d->name != NULL ? d->name : "(null)",
+                       bus_connection_get_loginfo (connection),
+                       limit);
+      return TRUE;
+    }
+
+  bus_context_log (d->connections->context, DBUS_SYSTEM_LOG_WARNING,
+      "Connection \"%s\" (%s) has had Unix fds pending for too long, "
+      "closing it (pending_fd_timeout=%d ms)",
+      d->name != NULL ? d->name : "(null)",
+      bus_connection_get_loginfo (connection),
+      limit);
+
   dbus_connection_close (connection);
   return TRUE;
 }
@@ -1012,6 +1043,7 @@ bus_connection_get_loginfo (DBusConnection        *connection)
   BusConnectionData *d;
     
   d = BUS_CONNECTION_DATA (connection);
+  _dbus_assert(d != NULL);
 
   if (!bus_connection_is_active (connection))
     return "inactive";
@@ -1248,8 +1280,9 @@ bus_connection_is_active (DBusConnection *connection)
   BusConnectionData *d;
 
   d = BUS_CONNECTION_DATA (connection);
+  _dbus_assert(d != NULL);
   
-  return d != NULL && d->name != NULL;
+  return d->name != NULL;
 }
 
 dbus_bool_t
@@ -2175,6 +2208,7 @@ bus_transaction_get_context (BusTransaction  *transaction)
 dbus_bool_t
 bus_transaction_capture (BusTransaction *transaction,
                          DBusConnection *sender,
+                         DBusConnection *addressed_recipient,
                          DBusMessage    *message)
 {
   BusConnections *connections;
@@ -2194,8 +2228,8 @@ bus_transaction_capture (BusTransaction *transaction,
    * There's little point, since there is up to 1 per process. */
   _dbus_assert (mm != NULL);
 
-  if (!bus_matchmaker_get_recipients (mm, connections, sender, NULL, message,
-        &recipients))
+  if (!bus_matchmaker_get_recipients (mm, connections, sender,
+        addressed_recipient, message, &recipients))
     goto out;
 
   for (link = _dbus_list_get_first_link (&recipients);
@@ -2217,6 +2251,7 @@ out:
 
 dbus_bool_t
 bus_transaction_capture_error_reply (BusTransaction  *transaction,
+                                     DBusConnection  *addressed_recipient,
                                      const DBusError *error,
                                      DBusMessage     *in_reply_to)
 {
@@ -2243,7 +2278,7 @@ bus_transaction_capture_error_reply (BusTransaction  *transaction,
   if (!dbus_message_set_sender (reply, DBUS_SERVICE_DBUS))
     goto out;
 
-  ret = bus_transaction_capture (transaction, NULL, reply);
+  ret = bus_transaction_capture (transaction, NULL, addressed_recipient, reply);
 
 out:
   dbus_message_unref (reply);
@@ -2285,7 +2320,7 @@ bus_transaction_send_from_driver (BusTransaction *transaction,
   /* Capture it for monitors, even if the real recipient's receive policy
    * does not allow it to receive this message from us (which would be odd).
    */
-  if (!bus_transaction_capture (transaction, NULL, message))
+  if (!bus_transaction_capture (transaction, NULL, connection, message))
     return FALSE;
 
   /* If security policy doesn't allow the message, we would silently
@@ -2297,7 +2332,8 @@ bus_transaction_send_from_driver (BusTransaction *transaction,
                                           transaction,
                                           NULL, connection, connection, message, &error))
     {
-      if (!bus_transaction_capture_error_reply (transaction, &error, message))
+      if (!bus_transaction_capture_error_reply (transaction, connection,
+                                                &error, message))
         {
           bus_context_log (transaction->context, DBUS_SYSTEM_LOG_WARNING,
                            "message from dbus-daemon rejected but not enough "
@@ -2649,6 +2685,8 @@ bus_connection_get_peak_match_rules (DBusConnection *connection)
   BusConnectionData *d;
 
   d = BUS_CONNECTION_DATA (connection);
+  _dbus_assert(d != NULL);
+
   return d->peak_match_rules;
 }
 
@@ -2658,6 +2696,8 @@ bus_connection_get_peak_bus_names (DBusConnection *connection)
   BusConnectionData *d;
 
   d = BUS_CONNECTION_DATA (connection);
+  _dbus_assert(d != NULL);
+
   return d->peak_bus_names;
 }
 #endif /* DBUS_ENABLE_STATS */
@@ -2668,8 +2708,9 @@ bus_connection_is_monitor (DBusConnection *connection)
   BusConnectionData *d;
 
   d = BUS_CONNECTION_DATA (connection);
+  _dbus_assert(d != NULL);
 
-  return d != NULL && d->link_in_monitors != NULL;
+  return d->link_in_monitors != NULL;
 }
 
 static dbus_bool_t
