@@ -25,12 +25,17 @@
 #ifdef DBUS_ENABLE_EMBEDDED_TESTS
 
 #include "dbus-auth-script.h"
+
+#include <stdio.h>
+
 #include "dbus-auth.h"
 #include "dbus-string.h"
 #include "dbus-hash.h"
 #include "dbus-credentials.h"
 #include "dbus-internals.h"
 
+#include "test/test-utils.h"
+
 /**
  * @defgroup DBusAuthScript code for running unit test scripts for DBusAuth
  * @ingroup  DBusInternals
@@ -221,10 +226,15 @@ auth_set_unix_credentials(DBusAuth  *auth,
     _dbus_assert_not_reached ("no memory");
 
   if (uid != DBUS_UID_UNSET)
-    _dbus_credentials_add_unix_uid (credentials, uid);
+    {
+      if (!_dbus_credentials_add_unix_uid (credentials, uid))
+        _dbus_assert_not_reached ("no memory");
+    }
   if (pid != DBUS_PID_UNSET)
-    _dbus_credentials_add_pid (credentials, pid);
-
+    {
+      if (!_dbus_credentials_add_pid (credentials, pid))
+        _dbus_assert_not_reached ("no memory");
+    }
   _dbus_auth_set_credentials (auth, credentials);
 
   _dbus_credentials_unref (credentials);
@@ -334,7 +344,7 @@ _dbus_auth_script_run (const DBusString *filename)
                                                "UNIX_ONLY"))
         {
           /* skip this file */
-          _dbus_warn ("skipping unix only auth script\n");
+          fprintf (stderr, "skipping unix only auth script\n");
           retval = TRUE;
           goto out;
         }
@@ -350,7 +360,7 @@ _dbus_auth_script_run (const DBusString *filename)
                                                "WIN_ONLY"))
         {
           /* skip this file */
-          _dbus_warn ("skipping windows only auth script\n");
+          fprintf (stderr, "skipping windows only auth script\n");
           retval = TRUE;
           goto out;
         }
@@ -506,9 +516,43 @@ _dbus_auth_script_run (const DBusString *filename)
           /* Replace USERID_HEX with our username in hex */
           {
             int where;
-            
-            if (_dbus_string_find (&to_send, 0,
-                                   "USERID_HEX", &where))
+
+            if (_dbus_string_find (&to_send, 0, "WRONG_USERID_HEX", &where))
+              {
+                /* This must be checked for before USERID_HEX, because
+                 * that's a substring. */
+                DBusString uid;
+
+                if (!_dbus_string_init (&uid))
+                  {
+                    _dbus_warn ("no memory for uid");
+                    _dbus_string_free (&to_send);
+                    goto out;
+                  }
+
+                if (!_dbus_test_append_different_uid (&uid))
+                  {
+                    _dbus_warn ("no memory for uid");
+                    _dbus_string_free (&to_send);
+                    _dbus_string_free (&uid);
+                    goto out;
+                  }
+
+                _dbus_string_delete (&to_send, where,
+                                     (int) strlen ("WRONG_USERID_HEX"));
+
+                if (!_dbus_string_hex_encode (&uid, 0, &to_send, where))
+                  {
+                    _dbus_warn ("no memory to subst WRONG_USERID_HEX");
+                    _dbus_string_free (&to_send);
+                    _dbus_string_free (&uid);
+                    goto out;
+                  }
+
+                _dbus_string_free (&uid);
+              }
+            else if (_dbus_string_find (&to_send, 0,
+                                        "USERID_HEX", &where))
               {
                 DBusString username;
 
@@ -527,7 +571,7 @@ _dbus_auth_script_run (const DBusString *filename)
                     goto out;
                   }
 
-                _dbus_string_delete (&to_send, where, strlen ("USERID_HEX"));
+                _dbus_string_delete (&to_send, where, (int) strlen ("USERID_HEX"));
                 
                 if (!_dbus_string_hex_encode (&username, 0,
 					      &to_send, where))
@@ -540,6 +584,51 @@ _dbus_auth_script_run (const DBusString *filename)
 
                 _dbus_string_free (&username);
               }
+            else if (_dbus_string_find (&to_send, 0,
+                                        "WRONG_USERNAME_HEX", &where))
+              {
+                /* This must be checked for before USERNAME_HEX, because
+                 * that's a substring. */
+#ifdef DBUS_UNIX
+                DBusString username;
+
+                if (!_dbus_string_init (&username))
+                  {
+                    _dbus_warn ("no memory for username");
+                    _dbus_string_free (&to_send);
+                    goto out;
+                  }
+
+                if (!_dbus_test_append_different_username (&username))
+                  {
+                    _dbus_warn ("no memory for username");
+                    _dbus_string_free (&to_send);
+                    _dbus_string_free (&username);
+                    goto out;
+                  }
+
+                _dbus_string_delete (&to_send, where,
+                                     (int) strlen ("WRONG_USERNAME_HEX"));
+
+                if (!_dbus_string_hex_encode (&username, 0,
+                                              &to_send, where))
+                  {
+                    _dbus_warn ("no memory to subst WRONG_USERNAME_HEX");
+                    _dbus_string_free (&to_send);
+                    _dbus_string_free (&username);
+                    goto out;
+                  }
+
+                _dbus_string_free (&username);
+#else
+                /* No authentication mechanism uses the login name on
+                 * Windows, so there's no point in it appearing in an
+                 * auth script that is not UNIX_ONLY. */
+                _dbus_warn ("WRONG_USERNAME_HEX cannot be used on Windows");
+                _dbus_string_free (&to_send);
+                goto out;
+#endif
+              }
             else if (_dbus_string_find (&to_send, 0,
                                         "USERNAME_HEX", &where))
               {
@@ -560,7 +649,7 @@ _dbus_auth_script_run (const DBusString *filename)
                     goto out;
                   }
 
-                _dbus_string_delete (&to_send, where, strlen ("USERNAME_HEX"));
+                _dbus_string_delete (&to_send, where, (int) strlen ("USERNAME_HEX"));
                 
                 if (!_dbus_string_hex_encode (&username, 0,
 					      &to_send, where))
