@@ -230,8 +230,8 @@ main (int argc, char *argv[])
   DBusConnection *connection;
   DBusError error;
   DBusMessage *message;
-  int print_reply;
-  int print_reply_literal;
+  dbus_bool_t print_reply;
+  dbus_bool_t print_reply_literal;
   int reply_timeout;
   DBusMessageIter iter;
   int i;
@@ -464,6 +464,7 @@ main (int argc, char *argv[])
       DBusMessageIter container_iter;
 
       type = DBUS_TYPE_INVALID;
+      secondary_type = DBUS_TYPE_INVALID;
       arg = argv[i++];
       c = strchr (arg, ':');
 
@@ -544,6 +545,7 @@ main (int argc, char *argv[])
 	}
       else if (container_type == DBUS_TYPE_DICT_ENTRY)
 	{
+	  _dbus_assert (secondary_type != DBUS_TYPE_INVALID);
 	  append_dict (target_iter, type, secondary_type, c);
 	}
       else
